@@ -1,4 +1,4 @@
-# $Id: TLPDB.pm 48144 2018-07-05 18:05:05Z karl $
+# $Id: TLPDB.pm 48874 2018-10-10 02:42:50Z preining $
 # TeXLive::TLPDB.pm - tlpdb plain text database files.
 # Copyright 2007-2018 Norbert Preining
 # This file is licensed under the GNU General Public License version 2
@@ -6,7 +6,7 @@
 
 package TeXLive::TLPDB;
 
-my $svnrev = '$Revision: 48144 $';
+my $svnrev = '$Revision: 48874 $';
 my $_modulerevision = ($svnrev =~ m/: ([0-9]+) /) ? $1 : "unknown";
 sub module_revision { return $_modulerevision; }
 
@@ -287,7 +287,9 @@ sub from_file {
     if ($rootpath =~ m,file://*(.*)$,) {
       $rootpath = "/$1";
     }
-    if (-d "$rootpath/texmf-dist/web2c") {
+    if ($params{'media'}) {
+      $media = $params{'media'};
+    } elsif (-d "$rootpath/texmf-dist/web2c") {
       $media = 'local_uncompressed';
     } elsif (-d "$rootpath/texmf/web2c") { # older
       $media = 'local_uncompressed';
