@@ -15,12 +15,12 @@ $mirrors = {
       'http://mirror.lzu.edu.cn/CTAN/' => 1,
       'http://mirrors.cqu.edu.cn/CTAN/' => 1,
       'http://mirrors.geekpie.club/CTAN/' => 1,
-      'http://mirrors.shu.edu.cn/CTAN/' => 1,
-      'http://mirrors.tuna.tsinghua.edu.cn/CTAN/' => 1,
-      'http://mirrors.ustc.edu.cn/CTAN/' => 1
+      'http://mirrors.huaweicloud.com/repository/toolkit/CTAN/' => 1,
+      'http://mirrors.tuna.tsinghua.edu.cn/CTAN/' => 1
     },
     'India' => {
-      'http://ctan.imsc.res.in/' => 1
+      'http://ctan.imsc.res.in/' => 1,
+      'http://mirror.iopb.res.in/tex-archive/' => 1
     },
     'Iran' => {
       'http://ctan.asis.io/' => 1,
@@ -29,7 +29,6 @@ $mirrors = {
       'http://repo.iut.ac.ir/tex-archive/' => 1
     },
     'Japan' => {
-      'ftp://ftp.kddilabs.jp/CTAN/' => 1,
       'ftp://ftp.u-aizu.ac.jp/pub/tex/CTAN/' => 1,
       'http://ftp.jaist.ac.jp/pub/CTAN/' => 1,
       'http://ftp.yz.yamagata-u.ac.jp/pub/CTAN/' => 1
@@ -45,8 +44,7 @@ $mirrors = {
       'http://mirror.pregi.net/tex-archive/' => 1
     },
     'Taiwan' => {
-      'ftp://ftp.ccu.edu.tw/pub/tex/' => 1,
-      'http://ftp.yzu.edu.tw/CTAN/' => 1
+      'ftp://ftp.ccu.edu.tw/pub/tex/' => 1
     },
     'Thailand' => {
       'http://mirror.kku.ac.th/CTAN/' => 1
@@ -72,6 +70,7 @@ $mirrors = {
     'France' => {
       'http://ctan.crest.fr/tex-archive/' => 1,
       'http://ctan.mines-albi.fr/' => 1,
+      'http://ctan.tetaneutral.net/' => 1,
       'http://distrib-coffee.ipsl.jussieu.fr/pub/mirrors/ctan/' => 1,
       'http://mirror.ibcp.fr/pub/CTAN/' => 1,
       'http://mirrors.ircam.fr/pub/CTAN/' => 1,
@@ -81,10 +80,11 @@ $mirrors = {
       'ftp://ftp.fu-berlin.de/tex/CTAN/' => 1,
       'ftp://ftp.rrzn.uni-hannover.de/pub/mirror/tex-archive/' => 1,
       'ftp://ftp.tu-chemnitz.de/pub/tex/' => 1,
-      'http://ctan.127001.ovh/' => 1,
+      'http://ctan.mirror.norbert-ruehl.de/' => 1,
       'http://ctan.space-pro.be/tex-archive/' => 1,
       'http://ftp.fau.de/ctan/' => 1,
       'http://mirror.informatik.hs-fulda.de/tex-archive/' => 1,
+      'http://packages.oth-regensburg.de/ctan/' => 1,
       'http://vesta.informatik.rwth-aachen.de/ftp/pub/mirror/ctan/' => 1
     },
     'Greece' => {
@@ -133,7 +133,6 @@ $mirrors = {
       'http://ftp.acc.umu.se/mirror/CTAN/' => 1
     },
     'Switzerland' => {
-      'http://mirror.switch.ch/ftp/mirror/tex/' => 1,
       'http://www.pirbot.com/mirrors/ctan/' => 1
     },
     'UK' => {
@@ -156,14 +155,12 @@ $mirrors = {
       'http://ctan.math.illinois.edu/' => 1,
       'http://ctan.math.washington.edu/tex-archive/' => 1,
       'http://ctan.mirrors.hoobly.com/' => 1,
-      'http://ctan.scidock.com/tex-archive/' => 1,
-      'http://ctan.sharelatex.com/tex-archive/' => 1,
       'http://mirror.hmc.edu/ctan/' => 1,
+      'http://mirror.las.iastate.edu/tex-archive/' => 1,
       'http://mirror.utexas.edu/ctan/' => 1,
       'http://mirrors.concertpass.com/tex-archive/' => 1,
       'http://mirrors.ibiblio.org/pub/mirrors/CTAN/' => 1,
-      'http://mirrors.rit.edu/CTAN/' => 1,
-      'http://mirrors.sorengard.com/ctan/' => 1
+      'http://mirrors.rit.edu/CTAN/' => 1
     }
   },
   'Oceania' => {
